@@ -1,12 +1,13 @@
 import os
 
 import requests
+from requests.auth import HTTPBasicAuth
 import json
 import logging
 from pathlib import Path
-from typing import List, Dict, BinaryIO
+from typing import List, Dict, BinaryIO, Optional, Union
 from .endpoints import Endpoint
-from .models import Project, Dataset, Image
+from .models import Project, Dataset, Image, Video
 from datagym.exceptions.exceptions import (APIException,
                                            InvalidTokenException,
                                            ClientException,
@@ -34,13 +35,16 @@ class Client:
     MAX_NUM_URLS_PER_UPLOAD = 50
     logger = logging.getLogger(__name__)
 
-    def __init__(self, api_key: str) -> None:
+    def __init__(self, api_key: str, base_url: str = 'https://app.datagym.ai/',
+                 basic_auth: Optional[HTTPBasicAuth] = None) -> None:
         """ Initializes DataGym Client instance
-
+        
+        :param base_url: URL to your Datagym instance
         :param str api_key: The API key of your organization
+        :param Optional(HTTPBasicAuth) basic_auth: The basic authentification (username, password) for Datagym.ai
 
         """
-        self._endpoint = Endpoint()
+        self._endpoint = Endpoint(base_path=base_url)
         """ An instance of :class:`.Endpoint`.
 
         Provides the endpoints of DataGym's API
@@ -54,7 +58,7 @@ class Client:
         """
 
         self.__api_key = api_key
-        self.__auth = {"Authorization": f'Token {api_key}'}
+        self.__basic_auth = basic_auth
 
     def __repr__(self):
         return f'Client(api_key="{self.__api_key}")'
@@ -66,15 +70,17 @@ class Client:
             self,
             method: str,
             endpoint: str,
-            headers: dict or None,
-            json: list or dict = None,
-            data: BinaryIO or dict = None
+            headers: Optional[dict] = None,
+            auth: Optional[HTTPBasicAuth] = None,
+            json: Optional[Union[list, dict]] = None,
+            data: Optional[Union[BinaryIO, dict]] = None
     ) -> requests.Response:
         """ Send a HTTP request to a DataGym endpoint
 
         :param str method: The HTTP method (ex. GET, POST, PUT, etc.)
         :param str endpoint: The DataGym API endpoint
         :param dict headers: The HTTP request header
+        :param HTTPBasicAuth auth: The basic authentification for this request.
         :param dict json: The request body as json
         :param dict data: The request body
 
@@ -87,16 +93,19 @@ class Client:
             if json:
                 return requests.request(method=method,
                                         url=self._endpoint.BASE_PATH + endpoint,
+                                        auth=auth,
                                         headers=headers,
                                         json=json)
             elif data:
                 return requests.request(method=method,
                                         url=self._endpoint.BASE_PATH + endpoint,
+                                        auth=auth,
                                         headers=headers,
                                         data=data)
             else:
                 return requests.request(method=method,
                                         url=self._endpoint.BASE_PATH + endpoint,
+                                        auth=auth,
                                         headers=headers)
         except requests.exceptions.ConnectionError as e:
             raise e
@@ -112,14 +121,14 @@ class Client:
         """
         if response.ok:
             return True
-        elif json.loads(response.content)["key"] in WARNING_KEYS:
-            ClientExceptionNonFatal(self._msg_builder, self.logger, **json.loads(response.content))
-            return False
         elif response.status_code == 500:
             if response.content:
                 raise APIException(self._msg_builder, **json.loads(response.content))
             else:
                 response.raise_for_status()
+        elif json.loads(response.content)["key"] in WARNING_KEYS:
+            ClientExceptionNonFatal(self._msg_builder, self.logger, **json.loads(response.content))
+            return False
         else:
             if response.content:
                 raise ClientException(self._msg_builder, **json.loads(response.content))
@@ -128,8 +137,9 @@ class Client:
 
     def _is_token_valid(self) -> bool:
         response = self._request(method="HEAD",
-                                 endpoint=self._endpoint.PROJECT,
-                                 headers=self.__auth)
+                                 endpoint=self._endpoint.project(token=self.__api_key),
+                                 auth=self.__basic_auth,
+                                 )
 
         if not response.ok:
             raise InvalidTokenException(msg_builder=self._msg_builder,
@@ -149,8 +159,8 @@ class Client:
 
         """
         response = self._request(method="GET",
-                                 endpoint=self._endpoint.PROJECT,
-                                 headers=self.__auth)
+                                 endpoint=self._endpoint.project(token=self.__api_key),
+                                 auth=self.__basic_auth)
 
         if self._response_valid(response):
             content = json.loads(response.content)
@@ -195,8 +205,8 @@ class Client:
 
         """
         response = self._request(method="GET",
-                                 endpoint=self._endpoint.DATASET,
-                                 headers=self.__auth)
+                                 endpoint=self._endpoint.dataset(token=self.__api_key),
+                                 auth=self.__basic_auth)
 
         if self._response_valid(response):
             content = json.loads(response.content)
@@ -226,14 +236,67 @@ class Client:
         :rtype: Dict
 
         """
-        endpoint = self._endpoint.export_labels(project_id)
+        endpoint = self._endpoint.export_labels(project_id, token=self.__api_key)
 
         response = self._request(method="GET",
                                  endpoint=endpoint,
-                                 headers=self.__auth)
+                                 auth=self.__basic_auth)
 
         if self._response_valid(response):
-            return json.loads(response.content)
+
+            export_dict = json.loads(response.content)
+
+            for media_export in export_dict[1:]:
+                if 'task_export_url' in media_export:
+                    # Remove the beginning to leave only endpoint
+                    inner_endpoint = media_export['task_export_url'].replace(self._endpoint.BASE_PATH, "")
+
+                    inner_response = self._request(method="GET",
+                                                   endpoint=inner_endpoint,
+                                                   auth=self.__basic_auth)
+
+                    if self._response_valid(inner_response):
+                        temp_label = json.loads(inner_response.content)
+
+                        media_export['labels'] = temp_label
+            return export_dict
+        else:
+            return dict()
+
+    def export_single_video_labels(self, project_id: str, video_id: str) -> Dict:
+        """ Export the labeled data from a specific Video
+
+        :param str project_id: The Project ID
+        :param str video_id: The Video ID
+        :returns: The labeled data as Dictionary
+        :rtype: Dict
+
+        """
+        endpoint = self._endpoint.export_labels(project_id, token=self.__api_key)
+
+        response = self._request(method="GET",
+                                 endpoint=endpoint,
+                                 auth=self.__basic_auth)
+
+        if self._response_valid(response):
+
+            export_dict = json.loads(response.content)
+
+            for media_export in export_dict[1:]:
+                if media_export['internal_media_ID'] == video_id:
+                    if 'task_export_url' in media_export:
+                        # Remove the beginning to leave only endpoint
+                        inner_endpoint = media_export['task_export_url'].replace(self._endpoint.BASE_PATH, "")
+                        inner_response = self._request(method="GET",
+                                                       endpoint=inner_endpoint,
+                                                       auth=self.__basic_auth)
+
+                        if self._response_valid(inner_response):
+                            temp_label = json.loads(inner_response.content)
+                            media_export['labels'] = temp_label
+                    return media_export
+
+            return dict()
 
     def export_labels_url(self, project_id: str) -> str:
         """ Generate a URL to the labeled data from a specific Project
@@ -247,11 +310,46 @@ class Client:
 
         response = self._request(method="HEAD",
                                  endpoint=endpoint,
-                                 headers=None)
+                                 auth=self.__basic_auth)
 
         if self._response_valid(response):
             return response.url
 
+    def download_video(self, video: Video, file_path: str) -> None:
+        """ Download a Video and store it in a specific file path
+
+        :param Video video: The Video Object to be downloaded
+        :param file_path: The destination path for storing the Video
+
+        """
+        video_content = self.download_video_bytes(video)
+        if video_content:
+            path_dir = Path(file_path)
+            path_file = path_dir.joinpath(video.video_name)
+            with open(path_file, 'wb') as handler:
+                handler.write(video_content)
+
+    def download_video_bytes(self, video: Video) -> bytes:
+        """ Download a Video and store it in a specific file path
+
+        :param Video video: The Video Object to be downloaded
+        :returns: Byte stream of the requested Video
+        :rtype: bytes
+
+        """
+        endpoint = self._endpoint.download_media(video.id, token=self.__api_key)
+
+        response = self._request(method="GET",
+                                 endpoint=endpoint,
+                                 auth=self.__basic_auth)
+
+        if self._response_valid(response):
+            inner_response = requests.request(method="GET",
+                                              url=response.content.decode("utf-8"))
+
+            if self._response_valid(inner_response):
+                return inner_response.content
+
     def download_image(self, image: Image, file_path: str) -> None:
         """ Download an Image and store it in a specific file path
 
@@ -259,14 +357,14 @@ class Client:
         :param file_path: The destination path for storing the Image
 
         """
-        endpoint = self._endpoint.download_image(image.id)
+        endpoint = self._endpoint.download_media(image.id, token=self.__api_key)
 
         path_dir = Path(file_path)
         path_file = path_dir.joinpath(image.image_name)
 
         response = self._request(method="GET",
                                  endpoint=endpoint,
-                                 headers=self.__auth)
+                                 auth=self.__basic_auth)
 
         if self._response_valid(response):
             with open(path_file, 'wb') as handler:
@@ -280,11 +378,11 @@ class Client:
         :rtype: bytes
 
         """
-        endpoint = self._endpoint.download_image(image.id)
+        endpoint = self._endpoint.download_media(image.id, token=self.__api_key)
 
         response = self._request(method="GET",
                                  endpoint=endpoint,
-                                 headers=self.__auth)
+                                 auth=self.__basic_auth)
 
         if self._response_valid(response):
             return response.content
@@ -309,18 +407,18 @@ class Client:
         headers = {
             'Content-type': 'application/json',
             "Accept": "application/json",
-            **self.__auth
         }
 
         response = self._request(method="POST",
-                                 endpoint=self._endpoint.DATASET,
+                                 endpoint=self._endpoint.dataset(token=self.__api_key),
                                  headers=headers,
+                                 auth=self.__basic_auth,
                                  json=data)
 
         if self._response_valid(response):
             return Dataset(json.loads(response.content))
-        else:   # In case of non-fatal exception
-            return self.get_dataset_by_name("name")
+        else:  # In case of non-fatal exception
+            return self.get_dataset_by_name(name)
 
     def add_dataset(self, dataset_id: str, project_id: str) -> bool:
         """ Add a Dataset to a Project
@@ -331,15 +429,15 @@ class Client:
         :rtype: bool
 
         """
-        endpoint = self._endpoint.add_dataset(project_id, dataset_id)
+        endpoint = self._endpoint.add_dataset(project_id, dataset_id, token=self.__api_key)
 
         response = self._request(method="POST",
                                  endpoint=endpoint,
-                                 headers=self.__auth)
+                                 auth=self.__basic_auth)
 
         if self._response_valid(response):
             return True
-        else:   # In case of non-fatal exception (already attached)
+        else:  # In case of non-fatal exception (already attached)
             return True
 
     def remove_dataset(self, dataset_id: str, project_id: str) -> bool:
@@ -351,11 +449,11 @@ class Client:
         :rtype: bool
 
         """
-        endpoint = self._endpoint.remove_dataset(project_id, dataset_id)
+        endpoint = self._endpoint.remove_dataset(project_id, dataset_id, token=self.__api_key)
 
         response = self._request(method="DELETE",
                                  endpoint=endpoint,
-                                 headers=self.__auth)
+                                 auth=self.__basic_auth)
 
         if self._response_valid(response):
             return True
@@ -372,13 +470,13 @@ class Client:
         :rtype: List[Dict[str, str]]
 
         """
-        endpoint = self._endpoint.create_image(dataset_id)
+        endpoint = self._endpoint.create_image(dataset_id, token=self.__api_key)
 
         if len(image_url_list) < self.MAX_NUM_URLS_PER_UPLOAD:
 
             response = self._request(method="POST",
                                      endpoint=endpoint,
-                                     headers=self.__auth,
+                                     auth=self.__basic_auth,
                                      json=image_url_list)
             if self._response_valid(response):
                 return json.loads(response.content)
@@ -394,11 +492,11 @@ class Client:
                 if i + self.MAX_NUM_URLS_PER_UPLOAD >= len(image_url_list):
                     slice = image_url_list[i:]
                 else:
-                    slice = image_url_list[i:i+self.MAX_NUM_URLS_PER_UPLOAD]
+                    slice = image_url_list[i:i + self.MAX_NUM_URLS_PER_UPLOAD]
 
                 partial_response = self._request(method="POST",
                                                  endpoint=endpoint,
-                                                 headers=self.__auth,
+                                                 auth=self.__basic_auth,
                                                  json=slice)
 
                 if self._response_valid(partial_response):
@@ -416,11 +514,11 @@ class Client:
         :rtype: bool
 
         """
-        endpoint = self._endpoint.delete_image(image.id)
+        endpoint = self._endpoint.delete_media(image.id, token=self.__api_key)
 
         response = self._request(method="DELETE",
                                  endpoint=endpoint,
-                                 headers=self.__auth)
+                                 auth=self.__basic_auth)
 
         if self._response_valid(response):
             return True
@@ -433,13 +531,13 @@ class Client:
         :param Dict label_data: Labels in JSON Format. See DataGym Docs for more information
         :returns: List of Errors if JSON is malformed or data is invalid
         """
-        endpoint = self._endpoint.import_labels(project_id=project_id)
+        endpoint = self._endpoint.import_labels(project_id=project_id, token=self.__api_key)
 
         if len(label_data) < self.MAX_NUM_URLS_PER_UPLOAD:
 
             response = self._request(method="POST",
                                      endpoint=endpoint,
-                                     headers=self.__auth,
+                                     auth=self.__basic_auth,
                                      json=label_data)
 
             if self._response_valid(response):
@@ -460,7 +558,7 @@ class Client:
 
                 partial_response = self._request(method="POST",
                                                  endpoint=endpoint,
-                                                 headers=self.__auth,
+                                                 auth=self.__basic_auth,
                                                  json=mini_batch)
 
                 if self._response_valid(partial_response):
@@ -470,18 +568,18 @@ class Client:
 
             return response
 
-    def upload_image(self, dataset_id: str, image_path: str, image_name: str = None) -> bool:
+    def upload_image(self, dataset_id: str, image_path: str, image_name: str = None) -> Image or None:
         """ Uploads an Image to a Dataset
 
         :param str dataset_id: The dataset the image should be uploaded to
         :param str image_path: The path to the image that should be uploaded
         :param str image_name: Your prefered image name.
                                 If left empty, it will automatically be extracted from the image path
-        :returns: True if Image was successfully uploaded
-        :rtype: bool
+        :returns: The Image if it was successfully uploaded, else None
+        :rtype: Image or None
 
         """
-        endpoint = self._endpoint.upload_image(dataset_id)
+        endpoint = self._endpoint.upload_media(dataset_id, token=self.__api_key)
 
         if not image_name:
             image_name = os.path.basename(image_path)
@@ -490,37 +588,37 @@ class Client:
 
         headers = {
             'X-filename': image_name,
-            **self.__auth
         }
 
         response = self._request(method="POST",
                                  endpoint=endpoint,
                                  headers=headers,
+                                 auth=self.__basic_auth,
                                  data=files)
 
         if self._response_valid(response):
             return Image(json.loads(response.content))
 
-    def upload_label_config(self, config_id: str, label_config: List[Dict]) -> bool:
+    def upload_label_config(self, config_id: str, label_config: List[Dict]) -> bytes or None:
         """ Clears the existing config and replaces it by a new one.
         Careful with this as clearing the config also clears all associated labels
 
         :param config_id:
         :param label_config:
-        :return: True if successful
+        :return: bytes if successful, else None
         """
-        clear_endpoint = self._endpoint.upload_label_config(config_id)
-        upload_endpoint = self._endpoint.clear_label_config(config_id)
+        clear_endpoint = self._endpoint.upload_label_config(config_id, token=self.__api_key)
+        upload_endpoint = self._endpoint.clear_label_config(config_id, token=self.__api_key)
 
         clear_response = self._request(method="DELETE",
                                        endpoint=clear_endpoint,
-                                       headers=self.__auth)
+                                       auth=self.__basic_auth)
 
         if self._response_valid(clear_response):
 
             upload_response = self._request(method="PUT",
                                             endpoint=upload_endpoint,
-                                            headers=self.__auth,
+                                            auth=self.__basic_auth,
                                             json=label_config)
 
             if self._response_valid(upload_response):
